package org.processmining.plugins.cosa.utils;

import java.io.IOException;
import java.io.InputStream;

public class PeekableInputStream extends InputStream {
	/**
	 * The underlying stream.
	 */
	private final InputStream stream;

	/**
	 * Bytes that have been peeked at.
	 */
	private byte peekBytes[];

	/**
	 * How many bytes have been peeked at.
	 */
	private int peekLength;

	/**
	 * The constructor accepts an InputStream to setup the object.
	 * 
	 * @param is
	 *            The InputStream to parse.
	 */
	public PeekableInputStream(InputStream is) {
		stream = is;
		peekBytes = new byte[10];
		peekLength = 0;
	}

	/**
	 * Peek at the next character from the stream.
	 * 
	 * @return The next character.
	 * @throws IOException
	 *             If an I/O exception occurs.
	 */
	public int peek() throws IOException {
		return peek(0);
	}

	/**
	 * Peek at a specified depth.
	 * 
	 * @param depth
	 *            The depth to check.
	 * @return The character peeked at.
	 * @throws IOException
	 *             If an I/O exception occurs.
	 */
	public int peek(int depth) throws IOException {
		// does the size of the peek buffer need to be extended?
		if (peekBytes.length <= depth) {
			byte temp[] = new byte[depth + 10];
			for (int i = 0; i < peekBytes.length; i++) {
				temp[i] = peekBytes[i];
			}
			peekBytes = temp;
		}

		// does more data need to be read?
		if (depth >= peekLength) {
			int offset = peekLength;
			int length = (depth - peekLength) + 1;
			int lengthRead = stream.read(peekBytes, offset, length);

			if (lengthRead == -1) {
				return -1;
			}

			peekLength = depth + 1;
		}

		return peekBytes[depth];
	}

	/*
	 * Read a single byte from the stream. @throws IOException If an I/O
	 * exception occurs. @return The character that was read from the stream.
	 */
	@Override
	public int read() throws IOException {
		if (peekLength == 0) {
			return stream.read();
		}

		int result = peekBytes[0];
		peekLength--;
		for (int i = 0; i < peekLength; i++) {
			peekBytes[i] = peekBytes[i + 1];
		}

		return result;
	}

}
